@@ -4,7 +4,7 @@ Created on Dec 13, 2013
 @author: joaograca
 '''
 
-
+import copy
 
 import requests
 import json
@@ -102,6 +102,7 @@ class Translation(object):
                  translators=[],
                  topics=None,
                  price=None,
+                 balance=None,
                  **kwargs):
         self.uid = uid
         self.text = text
@@ -112,12 +113,13 @@ class Translation(object):
         self.translators = translators
         self.topics = topics
         self.price = price
+        self.balance = balance
     
     def __repr__(self):
-        return "%s %s %s_%s"%(self.uid,self.status,self.source_language, self.target_language)
+        return "%s %s %s_%s_%s"%(self.uid,self.status,self.source_language, self.target_language, self.balance)
     
     def __str__(self):
-        return "%s %s %s_%s"%(self.uid,self.status,self.source_language, self.target_language)
+        return "%s %s %s_%s_%s"%(self.uid,self.status,self.source_language, self.target_language, self.balance)
 
 
 class UnbabelApi(object):
@@ -130,7 +132,9 @@ class UnbabelApi(object):
         self.username = username
         self.api_key = api_key
         self.api_url = api_url
+        self.is_bulk = False
     
+
     def post_translations(self,
                           text,
                           target_language,
@@ -142,58 +146,88 @@ class UnbabelApi(object):
                           callback_url = None,
                           topics = None,
                           instructions=None,
-                          uid=None,
+                          uid=None
                           ):
         
+
+        #data = self.create_default_translation(text, target_language)
+        data = {}
+        for k, v in locals().iteritems():
+            if v is self or v is data or v is None: 
+                continue
+            data[k] = v
+
+        if self.is_bulk:
+            self.bulk_data.append(data)
+            return
+
+        return self._make_request(data)
+
+    def _build_translation_object(self, json_object):
+        source_lang = json_object.get("source_language",None)
+        translation = json_object.get("translation",None)
+        status = json_object.get("status",None)
+            
+        translators = [Translator.from_json(t) for t in json_object.get("translators",[])]
+            
+        translation = Translation(
+            uid = json_object["uid"],
+            text = json_object["text"],
+            target_language = json_object.get('target_language', None),
+            source_language = json_object.get('source_lang', None),
+            translation = json_object.get('translation', None),
+            status = json_object.get('status', None),
+            translators = translators,
+            topics = json_object.get('topics', None),
+            price = json_object.get('price', None),
+            balance = json_object.get('balance', None)
+        )
+        return translation
+        
+
+    def _make_request(self, data):
+        
         headers={'Authorization': 'ApiKey %s:%s'%(self.username,self.api_key),'content-type': 'application/json'}
-        data = {
-                "text":text,
-                "target_language":target_language
-                }
-        if source_language:
-            data["source_language"] = source_language
-        if ttype:
-            data["type"] = ttype
-        if tone:
-            data["tone"] = tone
-        if visibility:
-            data["visibility"] = visibility
-        if public_url:
-            data["public_url"] = public_url
-        if callback_url:
-            data["callback_url"] = callback_url
-        if topics:
-            data["topics"] = topics
-        if instructions:
-            data["instructions"] = instructions
-        if uid:
-            data["uid"] = uid
-        result = requests.post("%stranslation/"%self.api_url,headers=headers,data=json.dumps(data))
-        if result.status_code == 201:
-            json_object =  json.loads(result.content)
-            source_lang = json_object.get("source_language",None)
-            translation = json_object.get("translation",None)
-            status = json_object.get("status",None)
-            
-            translators = [Translator.from_json(t) for t in json_object.get("translators",[])]
-            
-            
-            translation = Translation(uid=json_object["uid"],
-                                      text = json_object["text"],
-                                      target_language = target_language,
-                                      source_language = source_lang,
-                                      translation = translation,
-                                      status=status,
-                                      translators=translators,
-                                      topics = topics
-                                      )
-            return translation
+        if self.is_bulk:
+            f = requests.patch
+        else:
+            f = requests.post
+        result = f("%stranslation/"% self.api_url, headers=headers, data=json.dumps(data))
+        if result.status_code in (201, 202):
+            json_object = json.loads(result.content)
+            toret = None
+            if self.is_bulk:
+                toret = []
+                for obj in json_object['objects']:
+                    toret.append(self._build_translation_object(obj))
+            else:
+                toret = self._build_translation_object(json_object)
+            return toret
         elif result.status_code == 401:
             raise UnauthorizedException(result.content)
         elif result.status_code == 400:
             raise BadRequestException(result.content)
         else:
-            raise Exception("Unknown Error")
+            raise Exception("Unknown Error return status %d: %s", result.status_code, result.content[0:100])
+
+    def start_bulk_transaction(self):
+        self.bulk_data = []
+        self.is_bulk = True
+
+    def _post_bulk(self):
+        data = {'objects' : self.bulk_data}
+        return self._make_request(data=data)
+
+    def post_bulk_translations(self, translations):
+        self.start_bulk_transaction()
+        for obj in translations:
+            obj = copy.deepcopy(obj)
+            text, target_language = obj['text'], obj['target_language']
+            del obj['text']
+            del obj['target_language']
+            self.post_translations(text, target_language, **obj)
+
+        return self._post_bulk()
 
     def get_translations(self):
         '''
